@@ -5,11 +5,13 @@
 use std::borrow::Cow;
 use std::io::Write;
 
+use syn::ext::IdentExt;
+
 use crate::bindgen::cdecl;
 use crate::bindgen::config::{Config, Language};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use crate::bindgen::dependencies::Dependencies;
-use crate::bindgen::ir::{GenericParams, GenericPath, Path};
+use crate::bindgen::ir::{GenericArgument, GenericParams, GenericPath, Path};
 use crate::bindgen::library::Library;
 use crate::bindgen::monomorph::Monomorphs;
 use crate::bindgen::utilities::IterHelpers;
@@ -77,6 +79,7 @@ impl PrimitiveType {
                     "c_ulong" => (IntKind::Long, false),
                     "c_ulonglong" => (IntKind::LongLong, false),
                     "size_t" => (IntKind::SizeT, false),
+                    "RawFd" => (IntKind::Int, true),
 
                     "isize" | "intptr_t" => (IntKind::Size, true),
                     "usize" | "uintptr_t" => (IntKind::Size, false),
@@ -297,10 +300,7 @@ impl PrimitiveType {
     }
 
     fn can_cmp_order(&self) -> bool {
-        match *self {
-            PrimitiveType::Bool => false,
-            _ => true,
-        }
+        !matches!(*self, PrimitiveType::Bool)
     }
 
     fn can_cmp_eq(&self) -> bool {
@@ -308,25 +308,82 @@ impl PrimitiveType {
     }
 }
 
-// The `U` part of `[T; U]`
+/// Constant expressions.
+///
+/// Used for the `U` part of `[T; U]` and const generics. We support a very
+/// limited vocabulary here: only identifiers and literals.
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub enum ArrayLength {
+pub enum ConstExpr {
     Name(String),
     Value(String),
 }
 
-impl ArrayLength {
+impl ConstExpr {
     pub fn as_str(&self) -> &str {
-        match self {
-            ArrayLength::Name(ref string) | ArrayLength::Value(ref string) => string,
+        match *self {
+            ConstExpr::Name(ref string) | ConstExpr::Value(ref string) => string,
         }
     }
 
-    fn rename_for_config(&mut self, config: &Config) {
-        if let ArrayLength::Name(ref mut name) = self {
+    pub fn rename_for_config(&mut self, config: &Config) {
+        if let ConstExpr::Name(ref mut name) = self {
             config.export.rename(name);
         }
     }
+
+    pub fn load(expr: &syn::Expr) -> Result<Self, String> {
+        match *expr {
+            syn::Expr::Lit(syn::ExprLit { ref lit, .. }) => {
+                let val = match *lit {
+                    syn::Lit::Bool(syn::LitBool { value, .. }) => value.to_string(),
+                    syn::Lit::Int(ref len) => len.base10_digits().to_string(),
+                    syn::Lit::Byte(ref byte) => u8::to_string(&byte.value()),
+                    syn::Lit::Char(ref ch) => u32::to_string(&ch.value().into()),
+                    _ => return Err(format!("can't handle const expression {:?}", lit)),
+                };
+                Ok(ConstExpr::Value(val))
+            }
+            syn::Expr::Path(ref path) => {
+                let generic_path = GenericPath::load(&path.path)?;
+                Ok(ConstExpr::Name(generic_path.export_name().to_owned()))
+            }
+            _ => Err(format!("can't handle const expression {:?}", expr)),
+        }
+    }
+
+    pub fn specialize(&self, mappings: &[(&Path, &GenericArgument)]) -> ConstExpr {
+        match *self {
+            ConstExpr::Name(ref name) => {
+                let path = Path::new(name);
+                for &(param, value) in mappings {
+                    if path == *param {
+                        match *value {
+                            GenericArgument::Type(Type::Path(ref path))
+                                if path.is_single_identifier() =>
+                            {
+                                // This happens when the generic argument is a path.
+                                return ConstExpr::Name(path.name().to_string());
+                            }
+                            GenericArgument::Const(ref expr) => {
+                                return expr.clone();
+                            }
+                            _ => {
+                                // unsupported argument type - really should be an error
+                            }
+                        }
+                    }
+                }
+            }
+            ConstExpr::Value(_) => {}
+        }
+        self.clone()
+    }
+}
+
+impl Source for ConstExpr {
+    fn write<F: Write>(&self, _config: &Config, out: &mut SourceWriter<F>) {
+        write!(out, "{}", self.as_str());
+    }
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
@@ -342,11 +399,12 @@ pub enum Type {
     },
     Path(GenericPath),
     Primitive(PrimitiveType),
-    Array(Box<Type>, ArrayLength),
+    Array(Box<Type>, ConstExpr),
     FuncPtr {
         ret: Box<Type>,
         args: Vec<(Option<String>, Type)>,
         is_nullable: bool,
+        never_return: bool,
     },
 }
 
@@ -360,6 +418,22 @@ impl Type {
         }
     }
 
+    pub fn load_from_output(output: &syn::ReturnType) -> Result<(Type, bool), String> {
+        let mut never_return = false;
+        let ty = match output {
+            syn::ReturnType::Default => Type::Primitive(PrimitiveType::Void),
+            syn::ReturnType::Type(_, ref ty) => {
+                if let syn::Type::Never(_) = ty.as_ref() {
+                    never_return = true;
+                    Type::Primitive(PrimitiveType::Void)
+                } else {
+                    Type::load(ty)?.unwrap_or(Type::Primitive(PrimitiveType::Void))
+                }
+            }
+        };
+        Ok((ty, never_return))
+    }
+
     pub fn load(ty: &syn::Type) -> Result<Option<Type>, String> {
         let converted = match *ty {
             syn::Type::Reference(ref reference) => {
@@ -398,7 +472,7 @@ impl Type {
             syn::Type::Path(ref path) => {
                 let generic_path = GenericPath::load(&path.path)?;
 
-                if generic_path.name() == "PhantomData" {
+                if generic_path.name() == "PhantomData" || generic_path.name() == "PhantomPinned" {
                     return Ok(None);
                 }
 
@@ -412,28 +486,7 @@ impl Type {
                 }
             }
             syn::Type::Array(syn::TypeArray {
-                ref elem,
-                len: syn::Expr::Path(ref path),
-                ..
-            }) => {
-                let converted = Type::load(elem)?;
-
-                let converted = match converted {
-                    Some(converted) => converted,
-                    None => return Err("Cannot have an array of zero sized types.".to_owned()),
-                };
-                let generic_path = GenericPath::load(&path.path)?;
-                let len = ArrayLength::Name(generic_path.export_name().to_owned());
-                Type::Array(Box::new(converted), len)
-            }
-            syn::Type::Array(syn::TypeArray {
-                ref elem,
-                len:
-                    syn::Expr::Lit(syn::ExprLit {
-                        lit: syn::Lit::Int(ref len),
-                        ..
-                    }),
-                ..
+                ref elem, ref len, ..
             }) => {
                 let converted = Type::load(elem)?;
 
@@ -442,8 +495,7 @@ impl Type {
                     None => return Err("Cannot have an array of zero sized types.".to_owned()),
                 };
 
-                let len = ArrayLength::Value(len.base10_digits().to_string());
-                // panic!("panic -> value: {:?}", len);
+                let len = ConstExpr::load(len)?;
                 Type::Array(Box::new(converted), len)
             }
             syn::Type::BareFn(ref function) => {
@@ -461,7 +513,7 @@ impl Type {
                                             format!("_{}", wildcard_counter - 1)
                                         }
                                     } else {
-                                        ident.to_string()
+                                        ident.unraw().to_string()
                                     }
                                 }),
                                 ty,
@@ -469,21 +521,12 @@ impl Type {
                         })
                     })
                 })?;
-                let ret = match function.output {
-                    syn::ReturnType::Default => Type::Primitive(PrimitiveType::Void),
-                    syn::ReturnType::Type(_, ref ty) => {
-                        if let Some(x) = Type::load(ty)? {
-                            x
-                        } else {
-                            Type::Primitive(PrimitiveType::Void)
-                        }
-                    }
-                };
-
+                let (ret, never_return) = Type::load_from_output(&function.output)?;
                 Type::FuncPtr {
                     ret: Box::new(ret),
                     args,
                     is_nullable: false,
+                    never_return,
                 }
             }
             syn::Type::Tuple(ref tuple) => {
@@ -498,13 +541,14 @@ impl Type {
         Ok(Some(converted))
     }
 
+    pub fn is_ptr(&self) -> bool {
+        matches!(*self, Type::Ptr { .. } | Type::FuncPtr { .. })
+    }
+
     pub fn is_primitive_or_ptr_primitive(&self) -> bool {
         match *self {
             Type::Primitive(..) => true,
-            Type::Ptr { ref ty, .. } => match ty.as_ref() {
-                Type::Primitive(..) => true,
-                _ => false,
-            },
+            Type::Ptr { ref ty, .. } => matches!(ty.as_ref(), Type::Primitive(..)),
             _ => false,
         }
     }
@@ -543,10 +587,12 @@ impl Type {
                 ref ret,
                 ref args,
                 is_nullable: false,
+                never_return,
             } => Some(Type::FuncPtr {
                 ret: ret.clone(),
                 args: args.clone(),
                 is_nullable: true,
+                never_return,
             }),
             _ => None,
         }
@@ -602,7 +648,11 @@ impl Type {
             return None;
         }
 
-        let unsimplified_generic = &path.generics()[0];
+        let unsimplified_generic = match path.generics()[0] {
+            GenericArgument::Type(ref ty) => ty,
+            GenericArgument::Const(_) => return None,
+        };
+
         let generic = match unsimplified_generic.simplified_type(config) {
             Some(generic) => Cow::Owned(generic),
             None => Cow::Borrowed(unsimplified_generic),
@@ -630,7 +680,7 @@ impl Type {
                 is_ref: false,
             }),
             "Cell" => Some(generic.into_owned()),
-            "ManuallyDrop" | "MaybeUninit" if config.language != Language::Cxx => {
+            "ManuallyDrop" | "MaybeUninit" | "Pin" if config.language != Language::Cxx => {
                 Some(generic.into_owned())
             }
             _ => None,
@@ -638,18 +688,29 @@ impl Type {
     }
 
     pub fn simplify_standard_types(&mut self, config: &Config) {
+        self.visit_types(|ty| ty.simplify_standard_types(config));
         if let Some(ty) = self.simplified_type(config) {
             *self = ty;
         }
     }
 
     pub fn replace_self_with(&mut self, self_ty: &Path) {
+        if let Type::Path(ref mut generic_path) = *self {
+            generic_path.replace_self_with(self_ty);
+        }
+        self.visit_types(|ty| ty.replace_self_with(self_ty))
+    }
+
+    fn visit_types(&mut self, mut visitor: impl FnMut(&mut Type)) {
         match *self {
-            Type::Array(ref mut ty, ..) | Type::Ptr { ref mut ty, .. } => {
-                ty.replace_self_with(self_ty)
-            }
-            Type::Path(ref mut generic_path) => {
-                generic_path.replace_self_with(self_ty);
+            Type::Array(ref mut ty, ..) | Type::Ptr { ref mut ty, .. } => visitor(ty),
+            Type::Path(ref mut path) => {
+                for generic in path.generics_mut() {
+                    match *generic {
+                        GenericArgument::Type(ref mut ty) => visitor(ty),
+                        GenericArgument::Const(_) => {}
+                    }
+                }
             }
             Type::Primitive(..) => {}
             Type::FuncPtr {
@@ -657,9 +718,9 @@ impl Type {
                 ref mut args,
                 ..
             } => {
-                ret.replace_self_with(self_ty);
+                visitor(ret);
                 for arg in args {
-                    arg.1.replace_self_with(self_ty);
+                    visitor(&mut arg.1)
                 }
             }
         }
@@ -686,7 +747,7 @@ impl Type {
         }
     }
 
-    pub fn specialize(&self, mappings: &[(&Path, &Type)]) -> Type {
+    pub fn specialize(&self, mappings: &[(&Path, &GenericArgument)]) -> Type {
         match *self {
             Type::Ptr {
                 ref ty,
@@ -702,7 +763,9 @@ impl Type {
             Type::Path(ref generic_path) => {
                 for &(param, value) in mappings {
                     if generic_path.path() == param {
-                        return value.clone();
+                        if let GenericArgument::Type(ref ty) = *value {
+                            return ty.clone();
+                        }
                     }
                 }
 
@@ -717,13 +780,15 @@ impl Type {
                 Type::Path(specialized)
             }
             Type::Primitive(ref primitive) => Type::Primitive(primitive.clone()),
-            Type::Array(ref ty, ref constant) => {
-                Type::Array(Box::new(ty.specialize(mappings)), constant.clone())
-            }
+            Type::Array(ref ty, ref constant) => Type::Array(
+                Box::new(ty.specialize(mappings)),
+                constant.specialize(mappings),
+            ),
             Type::FuncPtr {
                 ref ret,
                 ref args,
                 is_nullable,
+                never_return,
             } => Type::FuncPtr {
                 ret: Box::new(ret.specialize(mappings)),
                 args: args
@@ -732,6 +797,7 @@ impl Type {
                     .map(|(name, ty)| (name, ty.specialize(mappings)))
                     .collect(),
                 is_nullable,
+                never_return,
             },
         }
     }
@@ -748,10 +814,12 @@ impl Type {
             }
             Type::Path(ref generic) => {
                 for generic_value in generic.generics() {
-                    generic_value.add_dependencies_ignoring_generics(generic_params, library, out);
+                    if let GenericArgument::Type(ref ty) = *generic_value {
+                        ty.add_dependencies_ignoring_generics(generic_params, library, out);
+                    }
                 }
                 let path = generic.path();
-                if !generic_params.contains(path) {
+                if !generic_params.iter().any(|param| param.name() == path) {
                     if let Some(items) = library.get_items(path) {
                         if !out.items.contains(path) {
                             out.items.insert(path.clone());
@@ -797,7 +865,7 @@ impl Type {
                 ty.add_monomorphs(library, out);
             }
             Type::Path(ref generic) => {
-                if generic.generics().is_empty() || out.contains(&generic) {
+                if generic.generics().is_empty() || out.contains(generic) {
                     return;
                 }
                 let path = generic.path();
@@ -884,7 +952,7 @@ impl Type {
                     return;
                 }
 
-                if let Some(mangled_path) = monomorphs.mangle_path(&generic_path) {
+                if let Some(mangled_path) = monomorphs.mangle_path(generic_path) {
                     *generic_path = GenericPath::new(mangled_path.clone(), vec![]);
                 } else {
                     warn!(
